package com.pj.project.relation_business_car;

import java.util.List;

import com.pj.utils.so.SoMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import com.pj.utils.sg.*;
import com.pj.project4sp.SP;

import com.pj.current.satoken.StpUserUtil;
import cn.dev33.satoken.annotation.SaCheckPermission;


/**
 * Controller: relation_business_car -- 
 * @author qzy 
 */
@RestController
@RequestMapping("/RelationBusinessCar/")
public class RelationBusinessCarController {

	/** 底层 Service 对象 */
	@Autowired
	RelationBusinessCarService relationBusinessCarService;

	/** 增 */  
	@RequestMapping("add")
	@SaCheckPermission(RelationBusinessCar.PERMISSION_CODE)
	@Transactional(rollbackFor = Exception.class)
	public AjaxJson add(RelationBusinessCar r){
		relationBusinessCarService.add(r);
		r = relationBusinessCarService.getById(SP.publicMapper.getPrimarykey());
		return AjaxJson.getSuccessData(r);
	}

	/** 删 */  
	@RequestMapping("delete")
	@SaCheckPermission(RelationBusinessCar.PERMISSION_CODE)
	public AjaxJson delete(Long id){
		relationBusinessCarService.delete(id);
		return AjaxJson.getSuccess();
	}
	
	/** 删 - 根据id列表 */  
	@RequestMapping("deleteByIds")
	@SaCheckPermission(RelationBusinessCar.PERMISSION_CODE)
	public AjaxJson deleteByIds(){
		List<Long> ids = SoMap.getRequestSoMap().getListByComma("ids", long.class); 
		int line = SP.publicMapper.deleteByIds(RelationBusinessCar.TABLE_NAME, ids);
		return AjaxJson.getByLine(line);
	}
	
	/** 改 */  
	@RequestMapping("update")
	@SaCheckPermission(RelationBusinessCar.PERMISSION_CODE)
	public AjaxJson update(RelationBusinessCar r){
		relationBusinessCarService.update(r);
		return AjaxJson.getSuccess();
	}

	/** 查 - 根据id */  
	@RequestMapping("getById")
	public AjaxJson getById(Long id){
		RelationBusinessCar r = relationBusinessCarService.getById(id);
		return AjaxJson.getSuccessData(r);
	}

	/** 查集合 - 根据条件（参数为空时代表忽略指定条件） */  
	@RequestMapping("getList")
	public AjaxJson getList() { 
		SoMap so = SoMap.getRequestSoMap();
		List<RelationBusinessCar> list = relationBusinessCarService.getList(so.startPage());
		return AjaxJson.getPageData(so.getDataCount(), list);
	}
	
	
	
	
	// ------------------------- 前端接口 -------------------------
	
	
	/** 改 - 不传不改 [G] */
	@RequestMapping("updateByNotNull")
	public AjaxJson updateByNotNull(Long id){
		AjaxError.throwBy(true, "如需正常调用此接口，请删除此行代码");
		// 鉴别身份，是否为数据创建者 
		long userId = SP.publicMapper.getColumnByIdToLong(RelationBusinessCar.TABLE_NAME, "user_id", id);
		AjaxError.throwBy(userId != StpUserUtil.getLoginIdAsLong(), "此数据您无权限修改");
		// 开始修改 (请只保留需要修改的字段)
		SoMap so = SoMap.getRequestSoMap();
		so.clearNotIn("id", "businessId", "businessCarId").clearNull().humpToLineCase();	
		int line = SP.publicMapper.updateBySoMapById(RelationBusinessCar.TABLE_NAME, so, id);
		return AjaxJson.getByLine(line);
	}
	
	
	
	
	
	

}
