package com.pj.project.tb_declare;

import java.util.List;

import com.pj.constants.UserTypeEnum;
import com.pj.utils.so.SoMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import com.pj.utils.sg.*;
import com.pj.project4sp.SP;

import com.pj.current.satoken.StpUserUtil;
import cn.dev33.satoken.annotation.SaCheckPermission;


/**
 * Controller: tb_declare -- 
 * @author lzm 
 */
@RestController
@RequestMapping("/TbDeclare/")
public class TbDeclareController {

	/** 底层 Service 对象 */
	@Autowired
	TbDeclareService tbDeclareService;

	/** 增 */  
	@RequestMapping("add")
	@SaCheckPermission(TbDeclare.PERMISSION_CODE)
	@Transactional(rollbackFor = Exception.class)
	public AjaxJson add(TbDeclare t){
		tbDeclareService.add(t);
		t = tbDeclareService.getById(SP.publicMapper.getPrimarykey());
		return AjaxJson.getSuccessData(t);
	}

	/** 删 */  
	@RequestMapping("delete")
	@SaCheckPermission(TbDeclare.PERMISSION_CODE)
	public AjaxJson delete(Long id){
		int line = tbDeclareService.delete(id);
		return AjaxJson.getByLine(line);
	}
	
	/** 删 - 根据id列表 */  
	@RequestMapping("deleteByIds")
	@SaCheckPermission(TbDeclare.PERMISSION_CODE)
	public AjaxJson deleteByIds(){
		List<Long> ids = SoMap.getRequestSoMap().getListByComma("ids", long.class);
		int line = SP.publicMapper.deleteByIds(TbDeclare.TABLE_NAME, ids);
		return AjaxJson.getByLine(line);
	}
	
	/** 改 */  
	@RequestMapping("update")
	@SaCheckPermission(TbDeclare.PERMISSION_CODE)
	public AjaxJson update(TbDeclare t){
		int line = tbDeclareService.update(t);
		return AjaxJson.getByLine(line);
	}

	/** 查 - 根据id */  
	@RequestMapping("getById")
	public AjaxJson getById(Long id){
		TbDeclare t = tbDeclareService.getById(id);
		return AjaxJson.getSuccessData(t);
	}

	/** 查集合 - 根据条件（参数为空时代表忽略指定条件） */  
	@RequestMapping("getList")
	public AjaxJson getList() { 
		SoMap so = SoMap.getRequestSoMap();
        String currentCustomerId = StpUserUtil.getCustomerId();
        if (!currentCustomerId.equals(UserTypeEnum.PLATFORM_ADMIN.getCustomerId())) {
            so.put("customerId", currentCustomerId);
        }
		List<TbDeclare> list = tbDeclareService.getList(so.startPage());
		return AjaxJson.getPageData(so.getDataCount(), list);
	}
	/** 查集合 - 根据条件（参数为空时代表忽略指定条件） */
	@RequestMapping("getNotBindList")
	public AjaxJson getNotBindList() {
		SoMap so = SoMap.getRequestSoMap();
		List<TbDeclare> list = tbDeclareService.getList(so.startPage());
		return AjaxJson.getPageData(so.getDataCount(), list);
	}
	
	
	
	
	// ------------------------- 前端接口 -------------------------
	
	
	/** 改 - 不传不改 [G] */
	@RequestMapping("updateByNotNull")
	public AjaxJson updateByNotNull(Long id){
		AjaxError.throwBy(true, "如需正常调用此接口，请删除此行代码");
		// 鉴别身份，是否为数据创建者 
		long userId = SP.publicMapper.getColumnByIdToLong(TbDeclare.TABLE_NAME, "user_id", id);
		AjaxError.throwBy(userId != StpUserUtil.getLoginIdAsLong(), "此数据您无权限修改");
		// 开始修改 (请只保留需要修改的字段)
		SoMap so = SoMap.getRequestSoMap();
		so.clearNotIn("id", "businessId", "goodsName", "grossWeight", "num", "productionDate", "expirationDate", "storageMode", "productionMode", "origin", "producerCode", "sendUnit", "receiveUnit", "route", "driverName", "driverPhone", "agent", "agentPhone", "chinaCarNo", "carNo", "customProof", "quarantineProof", "shop", "productionCode", "containerCode").clearNull().humpToLineCase();	
		int line = SP.publicMapper.updateBySoMapById(TbDeclare.TABLE_NAME, so, id);
		return AjaxJson.getByLine(line);
	}
	
	
	
	
	
	

}
