package com.pj.project.tb_discount;

import java.util.List;

import com.pj.project.tb_business.TbBusiness;
import com.pj.project.tb_business_car.TbBusinessCar;
import com.pj.project.tb_item.TbItem;
import com.pj.utils.so.SoMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import com.pj.utils.sg.*;
import com.pj.project4sp.SP;

import com.pj.current.satoken.StpUserUtil;
import cn.dev33.satoken.annotation.SaCheckPermission;


/**
 * Controller: tb_discount -- 折扣管理
 * @author qzy 
 */
@RestController
@RequestMapping("/TbDiscount/")
public class TbDiscountController {

	/** 底层 Service 对象 */
	@Autowired
	TbDiscountService tbDiscountService;

	/** 增 */  
	@RequestMapping("add")
	@SaCheckPermission(TbDiscount.PERMISSION_CODE)
	@Transactional(rollbackFor = Exception.class)
	public AjaxJson add(TbDiscount t){
		tbDiscountService.add(t);
		return AjaxJson.getSuccessData(t);
	}
	/** 改 - 状态(0=否,1=是) */
	@RequestMapping("updateStatus")
	@SaCheckPermission(TbDiscount.PERMISSION_CODE)
	public AjaxJson updateStatus(String id, int value){
		TbDiscount db=	tbDiscountService.getById(id);
		db.setStatus(value);
		tbDiscountService.updateById(db);
		return AjaxJson.getByLine(1);
	}
	/** 删 */  
	@RequestMapping("delete")
	@SaCheckPermission(TbDiscount.PERMISSION_CODE)
	public AjaxJson delete(Long id){
		int line = tbDiscountService.delete(id);
		return AjaxJson.getByLine(line);
	}
	
	/** 删 - 根据id列表 */  
	@RequestMapping("deleteByIds")
	@SaCheckPermission(TbDiscount.PERMISSION_CODE)
	public AjaxJson deleteByIds(){
		List<Long> ids = SoMap.getRequestSoMap().getListByComma("ids", long.class);
		int line = SP.publicMapper.deleteByIds(TbDiscount.TABLE_NAME, ids);
		return AjaxJson.getByLine(line);
	}
	
	/** 改 */  
	@RequestMapping("update")
	@SaCheckPermission(TbDiscount.PERMISSION_CODE)
	public AjaxJson update(TbDiscount t){
		int line = tbDiscountService.update(t);
		return AjaxJson.getByLine(line);
	}

	/** 查 - 根据id */  
	@RequestMapping("getById")
	public AjaxJson getById(Long id){
		TbDiscount t = tbDiscountService.getById(id);
		return AjaxJson.getSuccessData(t);
	}

	/** 查集合 - 根据条件（参数为空时代表忽略指定条件） */  
	@RequestMapping("getList")
	public AjaxJson getList() { 
		SoMap so = SoMap.getRequestSoMap();
		List<TbDiscount> list = tbDiscountService.getList(so.startPage());
		return AjaxJson.getPageData(so.getDataCount(), list);
	}
	
	
	
	
	// ------------------------- 前端接口 -------------------------
	
	
	/** 改 - 不传不改 [G] */
	@RequestMapping("updateByNotNull")
	public AjaxJson updateByNotNull(Long id){
		AjaxError.throwBy(true, "如需正常调用此接口，请删除此行代码");
		// 鉴别身份，是否为数据创建者 
		long userId = SP.publicMapper.getColumnByIdToLong(TbDiscount.TABLE_NAME, "user_id", id);
		AjaxError.throwBy(userId != StpUserUtil.getLoginIdAsLong(), "此数据您无权限修改");
		// 开始修改 (请只保留需要修改的字段)
		SoMap so = SoMap.getRequestSoMap();
		so.clearNotIn("id", "title", "type", "discount", "money", "status", "createTime").clearNull().humpToLineCase();	
		int line = SP.publicMapper.updateBySoMapById(TbDiscount.TABLE_NAME, so, id);
		return AjaxJson.getByLine(line);
	}
	
	
	
	
	
	

}
